"""Python functions to provide Django settings depending on other settings."""

import os
from typing import Any, Optional


def allauth_signup_form(values: dict[str, Any]) -> Optional[str]:
    """Return the form class to use for signing up."""
    if values.get("RECAPTCHA_PRIVATE_KEY") and values.get("RECAPTCHA_PUBLIC_KEY"):
        return "df_site.users.forms.ReCaptchaForm"
    return None


allauth_signup_form.required_settings = ["RECAPTCHA_PRIVATE_KEY", "RECAPTCHA_PUBLIC_KEY"]


def are_tests_running(values: dict[str, Any]) -> bool:
    """Return True if we are running unit tests."""
    return "testserver" in values.get("ALLOWED_HOSTS", [])


are_tests_running.required_settings = ["ALLOWED_HOSTS"]


def load_tox_environment():
    """Is a workaround for https://github.com/tox-dev/tox-docker/issues/55."""
    if os.environ.get("REDIS_HOST") and os.environ.get("REDIS_6379_TCP_PORT"):
        os.environ["REDIS_URL"] = f'redis://:p_df_site@{os.environ["REDIS_HOST"]}:{os.environ["REDIS_6379_TCP_PORT"]}/1'
    if os.environ.get("POSTGRES_HOST") and os.environ.get("POSTGRES_5432_TCP_PORT"):
        os.environ["DATABASE_URL"] = (
            f'postgresql://u_df_site:p_df_site@{os.environ["POSTGRES_HOST"]}:'
            f'{os.environ["POSTGRES_5432_TCP_PORT"]}/d_df_site'
        )
    if os.environ.get("MINIO_HOST") and os.environ.get("MINIO_9000_TCP_PORT"):
        os.environ["MAIN_STORAGE_DIR"] = (
            f's3:http://u_df_site:p_df_site@127.0.0.1:' f'{os.environ["MINIO_9000_TCP_PORT"]}/f_df_site'
        )



"""Admin classes for the df_site app."""

from functools import cached_property

from allauth.account.models import EmailAddress
from allauth.mfa.models import Authenticator
from allauth.socialaccount.models import SocialAccount
from allauth.usersessions.models import UserSession
from django.conf import settings
from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.utils.translation import gettext as _

from df_site.models import AlertRibbon, PreferencesUser


@admin.register(AlertRibbon)
class AlertRibbonAdmin(admin.ModelAdmin):
    """Admin class for the alert ribbon model."""

    list_display = ("message", "color", "start_date", "end_date", "is_active")
    list_filter = ("color", "start_date", "end_date", "is_active", "position")
    search_fields = ("summary",)
    fields = ["summary", "url", "message", "color", "start_date", "end_date", "is_active", "position"]


class EmailAddressInline(admin.TabularInline):
    """Inline for the email address model."""

    model = EmailAddress
    extra = 0


class SocialAccountInline(admin.TabularInline):
    """Inline for the social account model."""

    model = SocialAccount
    extra = 0
    fields = ["provider", "last_login", "date_joined"]
    readonly_fields = ["provider", "last_login", "date_joined"]

    def has_add_permission(self, request, obj):
        """Return False to prevent adding new social accounts."""
        return False


class AuthenticatorInline(admin.TabularInline):
    """Inline for the authenticator model."""

    model = Authenticator
    extra = 0
    fields = ["type", "created_at", "last_used_at"]
    readonly_fields = ["type", "created_at", "last_used_at"]

    def has_add_permission(self, request, obj):
        """Return False to prevent adding new authenticators."""
        return False


class UserSessionInline(admin.TabularInline):
    """Inline for the user session model."""

    model = UserSession
    extra = 0
    fields = [
        "created_at",
        "ip",
        "last_seen_at",
    ]
    readonly_fields = [
        "created_at",
        "ip",
        "last_seen_at",
    ]

    def has_add_permission(self, request, obj):
        """Return False to prevent adding new user sessions."""
        return False


@admin.register(PreferencesUser)
class PreferencesUserAdmin(UserAdmin):
    """Admin class for the preferences user model."""

    inlines = [EmailAddressInline, AuthenticatorInline, SocialAccountInline, UserSessionInline]
    list_display = ("username", "email", "first_name", "last_name", "is_staff", "date_joined")
    list_filter = ("is_staff", "is_superuser", "is_active", "date_joined", "groups")
    readonly_fields = ["last_login", "date_joined"]
    fieldsets = (
        (None, {"fields": ("username", "password")}),
        (
            _("Personal info"),
            {"fields": ("first_name", "last_name", "email", "color_theme", "display_online", "email_notifications")},
        ),
        (
            _("Permissions"),
            {
                "fields": (
                    "is_active",
                    "is_staff",
                    "is_superuser",
                    "groups",
                    "user_permissions",
                ),
            },
        ),
        (_("Important dates"), {"fields": (("last_login", "date_joined"),)}),
    )

    @cached_property
    def excluded_inlines(self):
        """Exclude inlines of which the corresponding app is not installed."""
        excluded = []
        if "allauth.mfa" not in settings.INSTALLED_APPS:
            excluded.append(AuthenticatorInline)
        if "allauth.socialaccount" not in settings.INSTALLED_APPS:
            excluded.append(SocialAccountInline)
        if "allauth.usersessions" not in settings.INSTALLED_APPS:
            excluded.append(UserSessionInline)
        if "allauth.account" not in settings.INSTALLED_APPS:
            excluded.append(EmailAddressInline)
        return excluded

    def get_inlines(self, request, obj):
        """Return inlines excluding those that correspond to apps not installed."""
        inlines = super().get_inlines(request, obj)
        if obj is None:
            return []
        inlines = [x for x in inlines if x not in self.excluded_inlines]
        return inlines
